% Posterior simulator for the following model of y_i i =1,n:
% p(y_i) = \sum_{j=1}^m  alpha(j) * \phi(y_i,mu_j,(hy*nuy(j))^-1) 
% Observables: y_ik, k=dd+1,...,d and y_ik \in [yda(k,i),ydb(k,i)], k=1,...,dd

function [sim_mu, sim_alphaunn, sim_hy, sim_nuy, sim_m] ...
         = post_simulator_optrjmcmc(y, yda, ydb,...
                            mu0, alphaunn0, hy0, nuy0, m0,...
                            mu_, Hmu_, A_, Ahy_, Bhy_, Anuy_, Bnuy_, Am_, Amlogp_,...
                            Nsim, jdt_flag, progress_step)


% some inputs check                                                      
szy = size(y); szyda = size(yda); szydb = size(ydb);
d = szy(1);
n = szy(2);
dd = szyda(1);
% if szyda(2) ~= n || szydb(2) ~= n || szydb(1) ~= dd  
%     error('post_simulator: input dimensions do not match');
% end


% %precompute some statistics used by the mcmc algorithm
% x1x1t = zeros(dx+1,dx+1,n);
% x1ty = zeros(dx+1,n);
% for i=1:n
%     x1x1t(:,:,i)=x1(:,i)*x1(:,i)';
%     x1ty(:,i)=x1(:,i)*y(i);
% end

mMaxInit = 20; % upper bound on m used for memory allocation before for loop, if m exceed this then more memory is allocated

hyAccCount = 0;
mp1AccCount = 0;
mm1AccCount = 0;



sim_mu = zeros(d,mMaxInit,Nsim);
sim_alphaunn = zeros(mMaxInit,Nsim);
sim_hy = zeros(d,Nsim); 
sim_nuy = zeros(d,mMaxInit,Nsim);
sim_m = zeros(1,Nsim);
sim_md = 0;
%sum_alphaunn = zeros(1,Nsim);
currlogLikelihood = zeros(1,Nsim); 
propPlogLikelihood = zeros(1,Nsim);
propMlogLikelihood = zeros(1,Nsim);

nonconvcount = 0;

mu=mu0; alphaunn=alphaunn0; hy=hy0; nuy=nuy0; m=m0;

tic


for sim = 1:Nsim


    
    [y,  mu, alphaunn, hy, nuy, m, hyAccCount] ...
         = mcmc_iter_given_m(y, yda, ydb, n, d, dd, ...
                                mu, alphaunn, hy, nuy, m,...
                                hyAccCount, ...
                                mu_, Hmu_, A_, Ahy_, Bhy_, Anuy_, Bnuy_, Am_, Amlogp_);
                            
                            
%      [s, mult_draw, N] = mcmc_iter_s_given_m_param(y, n, d, mu, alphaunn, hy, nuy, m);                       
%      [mu, alphaunn, hy, nuy, hyAccCount] = mcmc_iter_param_given_m_s_y(y, n, d, ...
%                                 s, mult_draw, N,...
%                                 hy, m,...
%                                 hyAccCount, ...
%                                 mu_, Hmu_, A_, Ahy_, Bhy_, Anuy_, Bnuy_);
%                             
    

    % store simulated params
%    sim_mu(:,1:m,sim) = mu; sim_alphaunn(1:m,sim) = alphaunn; sim_hy(:,sim) = hy; sim_nuy(:,1:m,sim) = nuy; sim_m(:,sim) = m;
    % %block for latent variables y
%     if dd
%         muy = mu(1:dd,s); sdy = 1./sqrt(hy(1:dd,1).*nuy(1:dd,s)); y(1:dd,:) = truncnormrnd(muy,sdy,yda,ydb);                            
%     end                            
   
    %%%%%% RJMCMC Step
    mdraw = randi(m); % random label permutation - valid mcmc transition
    if mdraw ~= m
        mutemp = mu(:,m); nuytemp = nuy(:,m); alphaunntemp = alphaunn(m);
        mu(:,m)=mu(:,mdraw); nuy(:,m) = nuy(:,mdraw); alphaunn(m)=alphaunn(mdraw);
        mu(:,mdraw)=mutemp; nuy(:,mdraw)=nuytemp; alphaunn(mdraw)=alphaunntemp;
    end

    FQ = FixedQuantInStruct_mcmc_m(y, hy, mu_, Hmu_, A_, Ahy_, Bhy_, Anuy_, Bnuy_, Am_, Amlogp_);
     
   
    [theta, m, AccFlag] = mcmc_iter_m([mu;nuy;alphaunn'], m, FQ);
    
    if AccFlag ~= 0
        mu = theta(1:d,:);
        nuy = theta(d+1:2*d,:);
        alphaunn = theta(2*d+1,:)';
    end
    
    if AccFlag == 1
        mp1AccCount = mp1AccCount + 1;
    elseif AccFlag == -1
        mm1AccCount = mm1AccCount + 1;
    end
    %%%%% End RJMCMC Step
    


    
    
    if m > mMaxInit % then need to allocate more memory for storage
        mMaxD = 5;
        sim_mu = cat(2, sim_mu, zeros(d,mMaxD,Nsim));
        sim_alphaunn = cat(1, sim_alphaunn, zeros(mMaxD,Nsim));
        sim_nuy = cat(2, sim_nuy, zeros(d,mMaxD,Nsim));
    end

% store simulated params
    sim_mu(:,1:m,sim) = mu;
    sim_alphaunn(1:m,sim) = alphaunn;
    sim_hy(:,sim) = hy; 
    sim_nuy(:,1:m,sim) = nuy; 
    sim_m(:,sim) = m;
    
    %alphaunn'
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %   sim
 %   y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    

    if jdt_flag % simulate data for joint d-n tests
        [y, yda, ydb] = gen_data(n, dd, mu, alphaunn, hy, nuy, m);
    end



    if floor(sim/progress_step) == sim/progress_step, % output current mcmc stats, e.g., acceptance rates
        msg = sprintf('Iter = %d Time = %d HyAccR = %d m = %d Params = %d %d %d %d', sim, toc, hyAccCount./sim, m, mu(1), alphaunn(1), hy(1), nuy(1,1));
        display(msg);
%         hy*nuy
         msg2 = sprintf('Acc Counts: mp1 = %d mm1 = %d hyAccR = %d ', mp1AccCount, mm1AccCount, hyAccCount./sim);
         display(msg2);
   
%          mu
%          nuy
%          hy
%          alphaunn
%        muAccCount
%        nuxAccCount
%        hxAccCount
%         N'
%      alphaunn
%       hyAccCount
        %plot_postsim; 
        drawnow;
    end
end

